@@ -243,6 +243,31 @@ module Graphics.Rendering.Cairo (
   , withSVGSurface
 #endif
 
+#if CAIRO_CHECK_VERSION(1,10,0)
+  -- * Regions
+  , regionCreate
+  , regionCreateRectangle
+  , regionCreateRectangles
+  , regionCopy
+  , regionGetExtents
+  , regionNumRectangles
+  , regionGetRectangle
+  , regionIsEmpty
+  , regionContainsPoint
+  , regionContainsRectangle
+  , regionEqual
+  , regionTranslate
+  , regionIntersect
+  , regionIntersectRectangle
+  , regionSubtract
+  , regionSubtractRectangle
+  , regionUnion
+  , regionUnionRectangle
+  , regionXor
+  , regionXorRectangle
+
+#endif
+
   -- * Utilities
 
   , liftIO
@@ -273,6 +298,11 @@ module Graphics.Rendering.Cairo (
   , HintMetrics(..)
   , FontOptions
   , Path
+#if CAIRO_CHECK_VERSION(1,10,0)
+  , RectangleInt(..)
+  , RegionOverlap(..)
+  , Region
+#endif
   , Content(..)
   , Format(..)
   , Extend(..)
@@ -1787,9 +1817,9 @@ imageSurfaceGetData a = do
 --   big-endian machine, the first pixel is in the uppermost bit, on a
 --   little-endian machine the first pixel is in the least-significant bit.
 --
--- * To read or write a specific pixel use the formula:
---   @p = y * (rowstride `div` 4) + x@ for the pixel and force the array to
---   have 32-bit words or integers.
+-- * To read or write a specific pixel (and assuming 'FormatARGB32' or
+--   'FormatRGB24'), use the formula: @p = y * (rowstride `div` 4) + x@ for the
+--   pixel and force the array to have 32-bit words or integers.
 --
 -- * Calling this function without explicitly giving it a type will often lead
 --   to a compiler error since the type parameter @e@ is underspecified. If
@@ -1801,28 +1831,20 @@ imageSurfaceGetData a = do
 --   the same type signatures as 'readArray' and 'writeArray'. Note that these
 --   are internal functions that might change with GHC.
 --
--- * After each write access to the array, you need to inform Cairo that
+-- * After each write access to the array, you need to inform Cairo
 --   about the area that has changed using 'surfaceMarkDirty'.
 --
 -- * The function will return an error if the surface is not an image
---   surface of if 'surfaceFinish' has been called on the surface.
+--   surface or if 'surfaceFinish' has been called on the surface.
 -- 
 imageSurfaceGetPixels :: Storable e => Surface -> IO (SurfaceData Int e)
 imageSurfaceGetPixels pb = do
-  pixPtr_ <- Internal.imageSurfaceGetData pb
-  when (pixPtr_==nullPtr) $ do
+  pixPtr <- Internal.imageSurfaceGetData pb
+  when (pixPtr==nullPtr) $ do
     fail "imageSurfaceGetPixels: image surface not available"
-  fmt <- imageSurfaceGetFormat pb
-  let bits = case fmt of
-               FormatARGB32 -> 32
-               FormatRGB24 -> 32
-               FormatA8 -> 8
-               FormatA1 -> 1
   h <- imageSurfaceGetHeight pb
   r <- imageSurfaceGetStride pb
-  let pixPtr = castPtr pixPtr_
-  let bytes = h*((r*bits)+7) `div` 8
-  return (mkSurfaceData pb pixPtr bytes)
+  return (mkSurfaceData pb (castPtr pixPtr) (h*r))
 
 -- | An array that stores the raw pixel data of an image 'Surface'.
 --
@@ -1833,7 +1855,7 @@ data SurfaceData i e = SurfaceData !Surface
 
 mkSurfaceData :: Storable e => Surface -> Ptr e -> Int -> SurfaceData Int e
 mkSurfaceData pb (ptr :: Ptr e) size =
-  SurfaceData pb ptr (0, count) count
+  SurfaceData pb ptr (0, count-1) count
   where count = fromIntegral (size `div` sizeOf (undefined :: e))
 
 #if __GLASGOW_HASKELL__ < 605
@@ -1985,6 +2007,208 @@ withSVGSurface filename width height f =
           (\surface -> f surface)
 #endif
 
+#if CAIRO_CHECK_VERSION(1,10,0)
+
+-- | Allocates a new empty region object.
+--
+regionCreate :: MonadIO m => m Region
+regionCreate = liftIO $ Internal.regionCreate
+
+-- | Allocates a new region object containing @rectangle@.
+--
+regionCreateRectangle ::
+     MonadIO m =>
+     RectangleInt -- ^ @rectangle@
+  -> m Region
+regionCreateRectangle a = liftIO $ Internal.regionCreateRectangle a
+
+-- | Allocates a new region object containing the union of all given @rects@.
+--
+regionCreateRectangles ::
+     MonadIO m =>
+     [RectangleInt] -- ^ @rects@
+  -> m Region
+regionCreateRectangles a = liftIO $ Internal.regionCreateRectangles a
+
+-- | Allocates a new region object copying the area from @original@.
+--
+regionCopy ::
+     MonadIO m =>
+     Region -- ^ @original@
+  -> m Region
+regionCopy a = liftIO $ Internal.regionCopy a
+
+-- | Gets the bounding rectangle of @region@ as a RectanglInt.
+--
+regionGetExtents ::
+     MonadIO m =>
+     Region -- ^ @region@
+  -> m RectangleInt
+regionGetExtents a = liftIO $ Internal.regionGetExtents a
+
+-- | Returns the number of rectangles contained in @region@.
+--
+regionNumRectangles ::
+     MonadIO m =>
+     Region -- ^ @region@
+  -> m Int
+regionNumRectangles a = liftIO $ Internal.regionNumRectangles a
+
+-- | Gets the @nth@ rectangle from the @region@.
+--
+regionGetRectangle ::
+     MonadIO m =>
+     Region -- ^ @region@
+  -> Int    -- ^ @nth@
+  -> m RectangleInt
+regionGetRectangle a n = liftIO $ Internal.regionGetRectangle a n
+
+-- | Checks whether @region@ is empty.
+--
+regionIsEmpty ::
+     MonadIO m =>
+     Region -- ^ @region@
+  -> m Bool
+regionIsEmpty a = liftIO $ Internal.regionIsEmpty a
+
+-- | Checks whether (@x@, @y@) is contained in @region@.
+--
+regionContainsPoint ::
+     MonadIO m =>
+     Region -- ^ @region@
+  -> Int    -- ^ @x@
+  -> Int    -- ^ @y@
+  -> m Bool
+regionContainsPoint a x y = liftIO $ Internal.regionContainsPoint a x y
+
+-- | Checks whether @rectangle@ is inside, outside or partially contained in @region@.
+--
+regionContainsRectangle ::
+     MonadIO m =>
+     Region       -- ^ @region@
+  -> RectangleInt -- ^ @rectangle@
+  -> m RegionOverlap
+regionContainsRectangle a rect = liftIO $ Internal.regionContainsRectangle a rect
+
+-- | Compares whether @region_a@ is equivalent to @region_b@.
+--
+regionEqual ::
+     MonadIO m =>
+     Region -- ^ @region_a@
+  -> Region -- ^ @region_b@
+  -> m Bool
+regionEqual a b = liftIO $ Internal.regionEqual a b
+
+-- | Translates @region@ by (@dx@, @dy@).
+--
+regionTranslate ::
+     MonadIO m =>
+     Region -- ^ @region@
+  -> Int    -- ^ @dx@
+  -> Int    -- ^ @dy@
+  -> m ()
+regionTranslate a dx dy = liftIO $ Internal.regionTranslate a dx dy
+
+-- | Computes the intersection of @dst@ with @other@ and places the result in @dst@.
+--
+regionIntersect ::
+     MonadIO m =>
+     Region -- ^ @dst@
+  -> Region -- ^ @other@
+  -> m ()
+regionIntersect a b = liftIO $ do
+  status <- Internal.regionIntersect a b
+  unless (status == StatusSuccess) $
+    Internal.statusToString status >>= fail
+
+-- | Computes the intersection of @dst@ with @rectangle@ and places the result in @dst@.
+--
+regionIntersectRectangle ::
+     MonadIO m =>
+     Region       -- ^ @dst@
+  -> RectangleInt -- ^ @rectangle@
+  -> m ()
+regionIntersectRectangle a rect = liftIO $ do
+  status <- Internal.regionIntersectRectangle a rect
+  unless (status == StatusSuccess) $
+    Internal.statusToString status >>= fail
+
+-- | Subtracts @other@ from @dst@ and places the result in @dst@.
+--
+regionSubtract ::
+     MonadIO m =>
+     Region -- ^ @dst@
+  -> Region -- ^ @other@
+  -> m ()
+regionSubtract a b = liftIO $ do
+  status <- Internal.regionSubtract a b
+  unless (status == StatusSuccess) $
+    Internal.statusToString status >>= fail
+
+-- | Subtracts @rectangle@ from @dst@ and places the result in @dst@.
+--
+regionSubtractRectangle ::
+     MonadIO m =>
+     Region       -- ^ @dst@
+  -> RectangleInt -- ^ @rectangle@
+  -> m ()
+regionSubtractRectangle a rect = liftIO $ do
+  status <- Internal.regionSubtractRectangle a rect
+  unless (status == StatusSuccess) $
+    Internal.statusToString status >>= fail
+
+-- | Computes the union of @dst@ with @other@ and places the result in @dst@.
+--
+regionUnion ::
+     MonadIO m =>
+     Region -- ^ @dst@
+  -> Region -- ^ @other@
+  -> m ()
+regionUnion a b = liftIO $ do
+  status <- Internal.regionUnion a b
+  unless (status == StatusSuccess) $
+    Internal.statusToString status >>= fail
+
+-- | Computes the union of @dst@ with @rectangle@ and places the result in @dst@.
+--
+regionUnionRectangle ::
+     MonadIO m =>
+     Region       -- ^ @dst@
+  -> RectangleInt -- ^ @rectangle@
+  -> m ()
+regionUnionRectangle a rect = liftIO $ do
+  status <- Internal.regionUnionRectangle a rect
+  unless (status == StatusSuccess) $
+    Internal.statusToString status >>= fail
+
+-- | Computes the exclusive difference of @dst@ with @other@ and places the result in @dst@.
+-- That is, @dst@ will be set to contain all areas that are either in @dst@ or in @other@, but not in both.
+--
+regionXor ::
+     MonadIO m =>
+     Region -- ^ @dst@
+  -> Region -- ^ @other@
+  -> m ()
+regionXor a b = liftIO $ do
+  status <- Internal.regionXor a b
+  unless (status == StatusSuccess) $
+    Internal.statusToString status >>= fail
+
+-- | Computes the exclusive difference of @dst@ with @rectangle@ and places the result in @dst@.
+-- That is, @dst@ will be set to contain all areas that are either in @dst@ or in @rectangle@, but not in both
+--
+regionXorRectangle ::
+     MonadIO m =>
+     Region       -- ^ @dst@
+  -> RectangleInt -- ^ @rectangle@
+  -> m ()
+regionXorRectangle a rect = liftIO $ do
+  status <- Internal.regionXorRectangle a rect
+  unless (status == StatusSuccess) $
+    Internal.statusToString status >>= fail
+
+#endif
+
 -- | Returns the version of the cairo library encoded in a single integer.
 --
 version :: Int
